@@ -353,7 +353,7 @@ class SOCol(object):
         if obj is None:
             # class attribute, return the descriptor itself
             return self
-        if obj.sqlmeta.obsolete:
+        if obj.sqlmeta._obsolete:
             raise '@@: figure out the exception for a delete'
         if obj.sqlmeta.cacheColumns:
             columns = obj.sqlmeta._columnCache
@@ -479,7 +479,7 @@ class SOStringLikeCol(SOCol):
         if self.customSQLType is not None:
             return self.customSQLType
         if not self.length:
-            if self.connection and self.connection.can_use_max_types:
+            if self.connection and self.connection.can_use_max_types():
                 type = 'VARCHAR(MAX)'
             else:
                 type = 'varchar(4000)'
@@ -523,7 +523,9 @@ class StringValidator(validators.Validator):
         if isinstance(value, str):
             return value
         if isinstance(value, unicode):
-            return value.encode("ascii")
+            connection = state.soObject._connection
+            dbEncoding = getattr(connection, "dbEncoding", None) or "ascii"
+            return value.encode(dbEncoding)
         return value
 
 class SOStringCol(SOStringLikeCol):
@@ -831,6 +833,7 @@ class SOForeignKey(SOKeyCol):
 
     def mysqlCreateReferenceConstraint(self):
         sTName = self.soClass.sqlmeta.table
+        sTLocalName = sTName.split('.')[-1]
         other = findClass(self.foreignKey, self.soClass.sqlmeta.registry)
         tName = other.sqlmeta.table
         idName = other.sqlmeta.idName
@@ -843,7 +846,7 @@ class SOForeignKey(SOKeyCol):
                 action = 'ON DELETE RESTRICT'
         else:
             action = ''
-        constraint = ('ALTER TABLE %(sTName)s ADD CONSTRAINT %(sTName)s_%(colName)s_exists '
+        constraint = ('ALTER TABLE %(sTName)s ADD CONSTRAINT %(sTLocalName)s_%(colName)s_exists '
                       'FOREIGN KEY (%(colName)s) '
                       'REFERENCES %(tName)s (%(idName)s) '
                       '%(action)s' %
@@ -851,7 +854,8 @@ class SOForeignKey(SOKeyCol):
                        'colName': self.dbName,
                        'idName': idName,
                        'action': action,
-                       'sTName': sTName})
+                       'sTName': sTName,
+                       'sTLocalName': sTLocalName})
         return constraint
 
     def mysqlCreateSQL(self):
@@ -936,8 +940,8 @@ class SOEnumCol(SOCol):
         return [consts.isString, consts.InList(self.enumValues)]
 
     def createValidators(self):
-        return [EnumValidator(name = self.name, enumValues = self.enumValues,
-                              notNone = self.notNone)] + \
+        return [EnumValidator(name=self.name, enumValues=self.enumValues,
+                              notNone=self.notNone)] + \
             super(SOEnumCol, self).createValidators()
 
     def _mysqlType(self):
@@ -1013,7 +1017,7 @@ class SOSetCol(SOCol):
         return [consts.isString, consts.InList(self.setValues)]
 
     def createValidators(self):
-        return [SetValidator(name = self.name, setValues = self.setValues)] + \
+        return [SetValidator(name=self.name, setValues=self.setValues)] + \
             super(SOSetCol, self).createValidators()
 
     def _mysqlType(self):
@@ -1276,10 +1280,7 @@ class TimestampCol(Col):
     baseClass = SOTimestampCol
 
 
-try:
-    from decimal import Decimal
-except ImportError:
-    Decimal = float
+from decimal import Decimal
 
 class DecimalValidator(validators.Validator):
     def to_python(self, value, state):
@@ -1332,7 +1333,7 @@ class SODecimalCol(SOCol):
         return 'DECIMAL(%i, %i)' % (self.size, self.precision)
 
     def createValidators(self):
-        return [DecimalValidator()] + \
+        return [DecimalValidator(name=self.name)] + \
             super(SODecimalCol, self).createValidators()
 
 class DecimalCol(Col):
@@ -1349,6 +1350,54 @@ class CurrencyCol(DecimalCol):
     baseClass = SOCurrencyCol
 
 
+class DecimalStringValidator(DecimalValidator):
+    def to_python(self, value, state):
+        value = super(DecimalStringValidator, self).to_python(value, state)
+        if self.precision and isinstance(value, Decimal):
+            assert value < self.max, \
+                    "Value must be less than %s" % int(self.max)
+            value = value.quantize(self.precision)
+        return value
+
+    def from_python(self, value, state):
+        value = super(DecimalStringValidator, self).from_python(value, state)
+        if isinstance(value, Decimal):
+            if self.precision:
+                assert value < self.max, \
+                        "Value must be less than %s" % int(self.max)
+                value = value.quantize(self.precision)
+            value = value.to_eng_string()
+        elif isinstance(value, (int, long)):
+            value = str(value)
+        return value
+
+class SODecimalStringCol(SOStringCol):
+    def __init__(self, **kw):
+        self.size = kw.pop('size', NoDefault)
+        assert (self.size is not NoDefault) and (self.size >= 0), \
+            "You must give a size argument as a positive integer"
+        self.precision = kw.pop('precision', NoDefault)
+        assert (self.precision is not NoDefault) and (self.precision >= 0), \
+               "You must give a precision argument as a positive integer"
+        kw['length'] = int(self.size) + int(self.precision)
+        self.quantize = kw.pop('quantize', False)
+        assert isinstance(self.quantize, bool), \
+                "quantize argument must be Boolean True/False"
+        super(SODecimalStringCol, self).__init__(**kw)
+
+    def createValidators(self):
+        if self.quantize:
+            v = DecimalStringValidator(
+                precision=Decimal(10) ** (-1 * int(self.precision)),
+                max=Decimal(10) ** (int(self.size) - int(self.precision)))
+        else:
+            v = DecimalStringValidator(precision=0)
+        return [v] + super(SODecimalStringCol, self).createValidators()
+
+class DecimalStringCol(StringCol):
+    baseClass = SODecimalStringCol
+
+
 class BinaryValidator(validators.Validator):
     """
     Validator for binary types.
@@ -1409,7 +1458,7 @@ class SOBLOBCol(SOStringCol):
         return 'BYTEA'
 
     def _mssqlType(self):
-        if self.connection and self.connection.can_use_max_types:
+        if self.connection and self.connection.can_use_max_types():
             return 'VARBINARY(MAX)'
         else:
             return "IMAGE"
