@@ -7,15 +7,9 @@ from sqlobject.tests.dbtest import *
 
 class TestUnicode(SQLObject):
     count = IntCol(alternateID=True)
-    col1 = UnicodeCol(alternateID=True)
+    col1 = UnicodeCol(alternateID=True, length=100)
     col2 = UnicodeCol(dbEncoding='latin-1')
 
-try:
-    enumerate
-except NameError: # Python 2.2
-    def enumerate(lst):
-        return [(i, lst[i]) for i in range(len(lst))]
-
 data = [u'\u00f0', u'test', 'ascii test']
 items = []
 
@@ -23,6 +17,8 @@ def setup():
     global items
     items = []
     setupClass(TestUnicode)
+    if TestUnicode._connection.dbName == 'postgres':
+        TestUnicode._connection.query('SET client_encoding TO latin1')
     for i, n in enumerate(data):
         items.append(TestUnicode(count=i, col1=n, col2=n))
 
